//**********************************************************************
// 	Project: 
//	File: mjpeg_exp.v
// 	Description: test M-JPEG （GW_2A_18）
//	Author: WenX
//  Timestamp: 
//----------------------------------------------------------------------
// Code Revision History:
// Ver:		| Author 	| Mod. Date		| Changes Made:
// v1.0.0	|           	| xx/xx/2021	    | Initial version
//**********************************************************************


module mjpeg_exp(
// --- sym ---
	input    SYM_CLK,
// --- KEY ---
	input    KEY1,
// --- SW ----
	input    SW1,
    output img_done,
	output			outvalid,
	output [7:0]	dout
);

//**********************************************************************
// --- Parameter
//**********************************************************************
    localparam NUM_CYCLE = 128;
	localparam MEM_DEPTH = 64*64;
`define 	IMAGE_WIDTH 64	
//**********************************************************************
// --- Inter Signal Declaration
//**********************************************************************
    wire clk;
    // wire clk_100M;
	wire rstn;
	wire ce;


    assign clk = SYM_CLK;
    assign rstn = KEY1;
	assign ce=SW1;

    wire						inpvalid;

	

	reg pstart;
	reg [19:0] addr;
    wire [11:0] mem_addr;
    reg [9:0] cnt_cycle;

    reg [7:0] data_mem_R [0:MEM_DEPTH-1] /* synthesis syn_romstyle= "block_rom" */;
    reg [7:0] data_mem_G [0:MEM_DEPTH-1] /* synthesis syn_romstyle= "block_rom" */;
    reg [7:0] data_mem_B [0:MEM_DEPTH-1] /* synthesis syn_romstyle= "block_rom" */;
    reg [7:0] din_R;
    reg [7:0] din_G;
    reg [7:0] din_B;
//**********************************************************************
// --- Initial
//**********************************************************************

initial begin
           $readmemh("./data/data_R.dat",data_mem_R);
           $readmemh("./data/data_G.dat",data_mem_G);
           $readmemh("./data/data_B.dat",data_mem_B);
end

assign inpvalid  = (addr ==1'b0) ? 1'b0 :(addr>=MEM_DEPTH+1) ? 1'b0 : (cnt_cycle[6:5] != 0) ? 1'b1 : 1'b0 ;

    always @(posedge clk or negedge rstn) begin
        if(!rstn) cnt_cycle <= NUM_CYCLE-1;
        else if(ce) begin
            if(addr==16'h1300) cnt_cycle<= NUM_CYCLE-1;
			else 
            if(cnt_cycle == NUM_CYCLE-1 ) cnt_cycle <= 0;
            else if(cnt_cycle == NUM_CYCLE-1) cnt_cycle <= cnt_cycle;
            else cnt_cycle <= cnt_cycle + 1;
        end
    end

    always @(posedge clk or negedge rstn) begin
        if(!rstn) addr <= 0;
//        else if(ce & pstart) begin
        else if(ce) begin   
            if (addr==16'h1300)addr <= 0;
			else
                if(cnt_cycle == NUM_CYCLE-1 && addr == MEM_DEPTH+1+2*`IMAGE_WIDTH+3) addr <= 0;
            else if(cnt_cycle[6:5] != 0)
                    addr <= addr + 1;
        end
    end


	always @(posedge clk or negedge rstn) begin
        if(!rstn) pstart <= 1'b0;
        else if(ce) begin
            if(cnt_cycle == NUM_CYCLE-1) pstart <= 1'b1;
            else
                pstart <= 1'b0;
        end
    end
	
    always @(posedge clk or negedge rstn) begin
        if(!rstn) begin 
        din_R <= 0;
        din_G <= 0;
        din_B <= 0;
end
//        else if(ce & pstart) begin
        else if(ce ) begin
			if(cnt_cycle == NUM_CYCLE-1 && addr >= MEM_DEPTH) 
            begin
                din_R <= 0;
                din_G <= 0;
                din_B <= 0;
            end
            else if(cnt_cycle[6:5] != 0)
            begin
                din_R <= data_mem_R[mem_addr];
                din_G <= data_mem_G[mem_addr];
                din_B <= data_mem_B[mem_addr];
            end
        end
    end
assign mem_addr=addr[11:0];
//**********************************************************************
// --- Module: top 
// --- Description:	top module of project
//**********************************************************************
MJPEG_Encoder_Top  mjpeg_top(
    .clk(clk), 				// system clock
    .rstn(rstn),				// reset
    .DE (inpvalid),
    .data_in    ({din_R,din_G,din_B}),
    .img_out    (dout),	// output data
    .img_valid  (outvalid),
    .img_done   (img_done)
);

	
 
endmodule
